@@ -619,7 +619,7 @@ class NodeManager(base.CreateManager):
     def set_provision_state(
             self, node_uuid, state, configdrive=None, cleansteps=None,
             rescue_password=None, os_ironic_api_version=None,
-            global_request_id=None):
+            global_request_id=None, deploysteps=None):
         """Set the provision state for the node.
 
         :param node_uuid: The UUID or name of the node.
@@ -644,20 +644,28 @@ class NodeManager(base.CreateManager):
             the request.  If not specified, the client's default is used.
         :param global_request_id: String containing global request ID header
             value (in form "req-<UUID>") to use for the request.
-
-        :raises: InvalidAttribute if there was an error with the clean steps
+        :param deploysteps: The deploy steps as a list of deploy-step
+            dictionaries; each dictionary should have keys 'interface', 'step',
+            'priority', and optional key 'args'. This is optional and is
+            only valid when setting provision-state to 'active' or 'rebuild'.
+        :raises: InvalidAttribute if there was an error with the clean steps or
+            deploy steps
         :returns: The status of the request
         """
 
         path = "%s/states/provision" % node_uuid
         body = {'target': state}
         if configdrive:
-            if not isinstance(configdrive, dict):
+            if isinstance(configdrive, str):
                 if os.path.isfile(configdrive):
                     with open(configdrive, 'rb') as f:
                         configdrive = f.read()
-                if os.path.isdir(configdrive):
+                elif os.path.isdir(configdrive):
                     configdrive = utils.make_configdrive(configdrive)
+                else:
+                    raise ValueError('Config drive seems to refer to a file '
+                                     'or directory but this file/directory '
+                                     'does not exist: %s.' % configdrive)
 
             if isinstance(configdrive, bytes):
                 try:
@@ -671,6 +679,9 @@ class NodeManager(base.CreateManager):
         elif rescue_password:
             body['rescue_password'] = rescue_password
 
+        if deploysteps:
+            body['deploy_steps'] = deploysteps
+
         return self.update(path, body, http_method='PUT',
                            os_ironic_api_version=os_ironic_api_version,
                            global_request_id=global_request_id)
